""" CMF Tool for processing Paypal payments

Based on:

import urllib

p = {"cmd"           : "_cart",
     "upload"        : "1",
          ## TODO: put it on portal_ordermanager
     "business"      : "gael@pilotsystems.net",
     #     "email"         : "toto@example.com",
     #     "first_name"    : "toto",
     #     "last_name"     : "tutu",
     #     "address1"      : "66 rue de provence",
     #     "address2"      : "",
     #     "zip"           : "75009",
     #     "city"          : "Paris",
     #     "state"         : "",
     #     "country"       : "France",
          ## TODO: hardcoded!
     "currency_code" : "EUR",
     "invoice"       : "12345",
     "no_shipping"   : "1",
     "return"        : 'http://dev.ens-colloque.pilotsystems.net/toto',
     "cancal_return"        : 'http://dev.ens-colloque.pilotsystems.net/toto',
     "notify_url"        : 'http://dev.ens-colloque.pilotsystems.net/toto',
     'item_name_1' : 'Inscription',
     'amount_1' : '11.00',
     'quantity_1'  : '1'
          }
url = "https://www.sandbox.paypal.com/cgi-bin/webscr"

param = []
for k, v in p.items():
      param.append("%s=%s" % (k, urllib.quote_plus(v)))
      param = "&".join(param)

      print "%s?%s" % (url, param)

$Id$
"""
import base64
import hmac
import random
import sha
import urllib
import urllib2

from AccessControl import ClassSecurityInfo
from AccessControl import getSecurityManager
from AccessControl.SecurityManagement import newSecurityManager
from AccessControl.SecurityManagement import setSecurityManager
from AccessControl.User import UnrestrictedUser as BaseUnrestrictedUser
from App.class_init import default__class_init__ as InitializeClass
from DocumentTemplate.DT_Var import html_quote
from OFS.PropertyManager import PropertyManager
from OFS.SimpleItem import SimpleItem

try:
    from zope.interface import implements
    from zope.component import queryAdapter
    from zope.component import queryUtility
except ImportError:
    def implements( *a, **k ): pass 
    def queryUtility( *a, **kw): return kw.get( "default", None )
    def queryAdapter( *a, **k): return None

from Products.CMFCore.utils import UniqueObject
from Products.CMFCore.utils import getToolByName

from Products.CMFPaypalHelper.interfaces import IPaypalTool
from Products.CMFPaypalHelper.interfaces import IPaypalVerifyCallback
from Products.CMFPaypalHelper.interfaces import IPaypalProcessor
from Products.CMFPaypalHelper.permissions import ManagePortal
from Products.CMFPaypalHelper.permissions import PayViaPaypal

class UnrestrictedUser(BaseUnrestrictedUser):
    """Unrestricted user that still has an id.
    """
    def getId(self):
        """Return the ID of the user."""
        return self.getUserName()

def gen_key():
    key=''
    for i in xrange(16):
        key += chr(random.randint(0,255))
    return key

def log_data( source, **kw ):
    try:
        import datetime
        fp = file( "/tmp/paypal-ipn.csv", "a" )
        fp.write( ";".join( [datetime.datetime.now().isoformat(), source, ] + ['"%s=%s"' % ( k,v ) for k,v in kw.iteritems() ] ) )
        fp.write( "\n" )
        fp.close()
    except:
        pass

class PaypalTool(UniqueObject, PropertyManager, SimpleItem):
    """ This tool encapsulates paypal payment information.
    """
    implements(IPaypalTool)
    security = ClassSecurityInfo()

    plone_tool = 1
    meta_type = "CMF Paypal helper tool"
    id = "portal_paypal"
    title = "Paypal configuration"
    icon_file = 'paypal_tool_icon.gif'

    # Property defaults
    paypal_url_scheme = 'https'
    paypal_domain = 'www.sandbox.paypal.com'
    paypal_path = '/cgi-bin/webscr'
    paypal_account_email = 'account@example.com'
    validate_script = 'validatePaypalPayment'
    default_currency = 'EUR'
    default_return_path = '/index_html?portal_status_message=Payment+accepted.'
    default_cancel_path = '/index_html?portal_status_message=Payment+cancelled.'
    paypal_button_image = 'http://www.paypal.com/en_US/i/btn/x-click-but01.gif'
    paypal_button_alt = ("Make payments with PayPal - "
                         "it's fast, free and secure!")
    tool_key = '' # replaced during __init__

    _properties = (
        {'id': 'paypal_url_scheme', 'type': 'string', 'mode': 'w'},
        {'id': 'paypal_domain', 'type': 'string', 'mode': 'w'},
        {'id': 'paypal_path', 'type': 'string', 'mode': 'w'},
        {'id': 'paypal_account_email', 'type': 'string', 'mode': 'w'},
        {'id': 'validate_script', 'type': 'string', 'mode': 'w'},
        {'id': 'default_currency', 'type': 'string', 'mode': 'w'},
        {'id': 'default_return_path', 'type': 'string', 'mode': 'w'},
        {'id': 'default_cancel_path', 'type': 'string', 'mode': 'w'},
        {'id': 'paypal_button_image', 'type': 'string', 'mode': 'w'},
        {'id': 'paypal_button_alt', 'type': 'string', 'mode': 'w'},
        #{'id': 'tool_key', 'type': 'string', 'mode': 'r'}, # *Don't* display
    )

    manage_options = ( PropertyManager.manage_options
                     + SimpleItem.manage_options)

    def __init__(self, *args, **kw):
        self.tool_key = base64.encodestring(gen_key())

    security.declareProtected(PayViaPaypal, 'makePaypalButton')
    def makePaypalButton(self,
                         currency=None,
                         cart=None,
                         invoice=None,
                         return_url=None,
                         cancel_url=None,
                         custom=None,
                        ):
        """ See IPaypalTool.
        """
        if cart is None:
            cart = ()

        url_tool = getToolByName(self, 'portal_url')
        portal_url = url_tool()

        _q = html_quote

        paypal_account = _q(self.paypal_account_email)
        full_paypal_url = _q('%s://%s/%s' % (self.paypal_url_scheme,
                                            self.paypal_domain,
                                            self.paypal_path.lstrip('/')))
        if not currency:
            currency = _q(self.default_currency)

        if not return_url:
            return_url = '%s/%s' % (portal_url,
                                    self.default_return_path.lstrip('/'))
        return_url = _q(return_url)

        if not cancel_url:
            cancel_url = '%s/%s' % (portal_url,
                                    self.default_cancel_path.lstrip('/'))
        cancel_url = _q(cancel_url)

        notify_url = _q('%s/paypal_notification' % self.absolute_url())

        button_url = _q(self.paypal_button_image)
        button_text = _q(self.paypal_button_alt)

        cart_form_fields = []

        def _hiddenField(name, value):
            cart_form_fields.append(
                '<input type="hidden" name="%s" value="%s" />' % (name, value))

        counter = 1
        for item_name, price, quantity in cart:
            subs = {'counter': counter,
                    'name': _q(item_name),
                    'price': price,
                    'quantity': quantity,
                   }
            _hiddenField('item_name_%d' % counter, _q(item_name))
            _hiddenField('amount_%d' % counter, '%0.2f' % price)
            _hiddenField('quantity_%d' % counter, '%0d' % quantity)
            counter += 1

        if invoice:
            _hiddenField('invoice',_q(str(invoice)))

        if custom:
            _hiddenField('custom', _q(str(custom)))

        form = """\
<form name="_xclick" method="post" action="%(full_paypal_url)s" >
<input type="hidden" name="cmd" value="_cart" />
<input type="hidden" name="upload" value="1" />
<input type="hidden" name="no_shipping" value="1" />
<input type="hidden" name="business" value="%(paypal_account)s" />
<input type="hidden" name="return" value="%(return_url)s" />
<input type="hidden" name="cancel_url" value="%(cancel_url)s" />
<input type="hidden" name="currency_code" value="%(currency)s" />
<input type="hidden" name="notify_url" value="%(notify_url)s" />
%(cart)s
<input type="image" src="%(button_url)s" border="0" name="submit"
       alt="%(button_text)s" />
</form>""" % {'cart': '\n'.join(cart_form_fields),
              'full_paypal_url': full_paypal_url,
              'paypal_account': paypal_account,
              'currency': currency,
              'return_url': return_url,
              'cancel_url': cancel_url,
              'button_url': button_url,
              'notify_url': notify_url,
              'button_text' :button_text,
             }

        return form

    security.declareProtected(PayViaPaypal, 'makePaypalSubscriptionButton')
    def makePaypalSubscriptionButton(self,
                         terms=None,
                         currency=None,
                         item_name=None,
                         invoice=None,
                         return_url=None,
                         cancel_url=None,
                         custom=None,
                         automatic_recurring=True,
                         reattempt_xaction=True,
                         recurring_times=None,
                         return_method=2,
                         modify=1,
                        ):
        """ See IPaypalTool.
        """
        if terms is None:
            # the following means:
            # trial period 1: free for 7 Days
            # trial period 2: 5.- for 3 Weeks
            # subscribtion:   50.- every 11 Months (_not_ every month)
            # please see also the paypal API docs.
            terms = [( 0.0, 7, "D" ), ( 5.0, 3, 'W' ), ( 50.0, 11, "M" )]

        url_tool = getToolByName(self, 'portal_url')
        portal_url = url_tool()

        _q = html_quote

        paypal_account = _q(self.paypal_account_email)
        full_paypal_url = _q('%s://%s/%s' % (self.paypal_url_scheme,
                                            self.paypal_domain,
                                            self.paypal_path.lstrip('/')))
        if not currency:
            currency = _q(self.default_currency)

        if not return_url:
            return_url = '%s/%s' % (portal_url,
                                    self.default_return_path.lstrip('/'))
        return_url = _q(return_url)

        if not cancel_url:
            cancel_url = '%s/%s' % (portal_url,
                                    self.default_cancel_path.lstrip('/'))
        cancel_url = _q(cancel_url)

        notify_url = _q('%s/paypal_notification' % self.absolute_url())

        button_url = _q(self.paypal_button_image)
        button_text = _q(self.paypal_button_alt)

        form_fields = []

        def _hiddenField(name, value):
            form_fields.append(
                '<input type="hidden" name="%s" value="%s" />' % (name, value))

        if item_name is not None:
            _hiddenField('item_name', _q(item_name))

        # subscription payments will be automatically recurring if this is set.
        if automatic_recurring is True:
            _hiddenField('src', '1' )

        # failed payments will be reattempted if this is set. If not, a failing
        # payment will result in an end of subscription
        if reattempt_xaction is True:
            _hiddenField('sra', '1' )

        # For automatic recurring payments you can set a limit of how many times this
        # payment will be recurring, unlimited if omitted.
        if recurring_times is not None:
            _hiddenField('srt', _q(str(recurring_times)) )

        # Modify: 0=no modify, 1=modify _or_ create subscription, 2=modify only
        if modify is not None:
            _hiddenField('modify', _q(str(modify)) )

        # return_method
        # 1: GET, 2: POST, not set: depends :( (see paypal docs)
        if return_method is not None:
            _hiddenField('rm', _q(str(return_method)) )

        counter = 1
        for amount, period, unit in terms:
            if amount >= 0.0:
                _hiddenField('a%d' % counter, '%0.2f' % amount)
                _hiddenField('p%d' % counter, '%d' % period)
                _hiddenField('t%d' % counter, unit )
            counter += 1

        if invoice:
            _hiddenField('invoice',_q(str(invoice)))

        if custom:
            _hiddenField('custom', _q(str(custom)))

        form = """\
<form name="_xclick-subscriptions" method="post" action="%(full_paypal_url)s" >
<input type="hidden" name="cmd" value="_xclick-subscriptions" />
<input type="hidden" name="no_shipping" value="1" />
<input type="hidden" name="business" value="%(paypal_account)s" />
<input type="hidden" name="return" value="%(return_url)s" />
<input type="hidden" name="cancel_url" value="%(cancel_url)s" />
<input type="hidden" name="currency_code" value="%(currency)s" />
<input type="hidden" name="notify_url" value="%(notify_url)s" />
%(form_fields)s
<input type="image" src="%(button_url)s" border="0" name="submit"
       alt="%(button_text)s" />
</form>""" % {'form_fields': '\n'.join(form_fields),
              'full_paypal_url': full_paypal_url,
              'paypal_account': paypal_account,
              'currency': currency,
              'return_url': return_url,
              'cancel_url': cancel_url,
              'button_url': button_url,
              'notify_url': notify_url,
              'button_text' :button_text,
             }

        return form

    security.declarePublic('paypal_notification')
    def paypal_notification(self, REQUEST):
        """ Process a paypal notification callback.
        """
        form = REQUEST.form

        # SELETZ: LOG IPN data
        log_data( "paypal_notification", **dict(form) )
        #/SELETZ

        # Fetch values
        invoice = form.get('invoice', '')
        custom = form.get('custom', invoice)
        email = form.get('receiver_email', '(receiver_email missing)')
        scheme = self.paypal_url_scheme
        domain = self.paypal_domain
        path = self.paypal_path.lstrip('/')

        # Step 1: check transaction
        if email != self.paypal_account_email:
            log_data( "paypal_notification", error="account email" )
            raise RuntimeError("Receiver email is not valid " + email)

        # Step 2: resubmit the form to paypal, and see if paypal validate it
        callback = queryUtility(IPaypalVerifyCallback,
                                default=PaypalVerifyCallback( scheme, domain, path ))

        ok, data = callback(**form)
        if not ok:
            log_data( "paypal_notification", error="verification" )
            raise RuntimeError("Paypal didn't verify transaction "
                               "(return is %s)" % data)

        # Step 3: prepare call to callback
        url_tool = getToolByName(self, 'portal_url')
        portal = url_tool.getPortalObject()
        script = queryAdapter(portal, IPaypalProcessor)
        if script is None:
            script = getattr(portal, self.validate_script)

        # Create Manager security context
        tmp_user = UnrestrictedUser('manager', '', ['Manager'], '')
        tmp_user = tmp_user.__of__(self.acl_users) # or use appropriate context
        old_sm = getSecurityManager()
        try:
            newSecurityManager(None, tmp_user)
            script(verification_key=self._makeVerificationKey(custom),
                   payment_info=dict(form.items()),
                  )
        finally:
            setSecurityManager(old_sm)

    security.declarePublic('checkVerificationKey')
    def checkVerificationKey(self, item, key):
        """ See IPaypalTool.
        """
        return key == self._makeVerificationKey(item)

    security.declarePrivate("_makeVerificationKey")
    def _makeVerificationKey(self, item):
        """ Make a verification key.
        """
        hash_func = sha
        key = base64.decodestring(self.tool_key)
        sig_gen = hmac.new(key, item, hash_func)
        return base64.encodestring(sig_gen.digest())

    security.declareProtected(ManagePortal, 'updateProperties')
    def updateProperties(self, REQUEST):
        """ See IPaypalTool.
        """
        form = REQUEST.form
        for key in ['paypal_url_scheme',
                    'paypal_domain',
                    'paypal_path',
                    'paypal_account_email',
                    'validate_script',
                    'default_currency',
                    'default_return_path',
                    'default_cancel_path',
                    'paypal_button_image',
                    'paypal_button_alt']:
            if key in form:
                self._updateProperty(key, form[key])

        REQUEST.RESPONSE.redirect(
            '%s/paypal_prefs?portal_status_message='
            'Saved configuration.' % self.absolute_url())

InitializeClass(PaypalTool)

def gen_key():
    key=''
    for i in xrange(16):
        key += chr(random.randint(0,255))
    return key

class PaypalVerifyCallback:
    implements(IPaypalVerifyCallback)

    def __init__( self, scheme, domain, path):
        self.scheme = scheme
        self.domain = domain
        self.path = path

    def __call__(self, **kw):
        params = dict(kw)
        params['cmd'] = '_notify-validate'

        # SELETZ: LOG IPN data
        log_data( "PaypalVerifyCallback", **params )
        #/SELETZ

        params = urllib.urlencode(params)
        headers = {"Content-type": "application/x-www-form-urlencoded" }
        req = urllib2.Request("%s://%s/%s" % (self.scheme, self.domain, self.path), params,
                              headers)
        conn = urllib2.urlopen(req)
        data = conn.read()

        # SELETZ: LOG IPN data
        log_data( "PaypalVerifyCallback", reply=data )
        #/SELETZ

        return 'VERIFIED' in data, data
