""" CMFPaypalHelper tool interfaces

$Id: interfaces.py 44457 2007-06-24 11:22:02Z seletz $
"""
try:
    from zope.interface import Attribute
    from zope.interface import Interface
except ImportError:
    def Attribute( desc ): pass
    class Interface:
        pass

class IPaypalVerifyCallback(Interface):
    """ Utility for returning the form-being-verified to Paypal.
    """
    def __call__(**kw):
        """ Verify a request from Paypal.

        o Return (status, data) where 'status' is a boolean indicating
          success or failure, and 'data' is additional information about
          the reason.
        """

class IPaypalProcessor(Interface):
    """ Utility for processing the data passed by Paypal.
    """
    def __call__(verification_key, payment_info):
        """ Process a verified request from Paypal.

        o 'verification_key' is a hash of the 'invoice' value with the
          tool's tool key.

        o 'payment_info' is a dict containing the full set of data passed
          by Paypal.
        """

class IPaypalTool(Interface):
    """ Tool for processing Paypal payments and callbacks.
    """
    paypal_url_scheme = Attribute("""Paypal URL Scheme

    o URL scheme used to construct Paypal POST request.

    Default value: 'https'""")

    paypal_domain = Attribute("""Paypal Domain

    o Host / part used to construct Paypal POST request.

    Default value: 'www.sandbox.paypal.com'""")

    paypal_path = Attribute("""Paypal Path

    o path used to construct Paypal POST request.

    Default value: '/cgi-bin/webscr'""")

    paypal_account_email = Attribute("""Paypal Account E-mail

    o E-mail address of Paypal account being paid.

    Default value: 'account@example.com'""")

    validate_script = Attribute("""Validate Script ID

    o ID of script used to handle Paypal callback requests;  this
      script would then be used to record payment, enable access, etc.

    Default value: 'validatePaypalPayment'""")

    default_currency = Attribute("""Default Currency

    o Default currency used for Paypal transactions.

    Default value: 'EUR'""")

    default_return_path = Attribute("""Default Return Path

    o URL of the default page to which Paypal will redirect after
      successfully processing a payment.

    Default value: '/index_html?portal_status_message=Payment+accepted.'""")

    default_cancel_path = Attribute("""Default Cancel Path

    o URL of the default page to which Paypal will redirect after
      cancelling a payment.

    Default value: '/index_html?portal_status_message=Payment+cancelled.'""")

    paypal_button_image = Attribute("""Paypal Button Image

    o Image URL for the Paypal button.

    Default value: 'http://www.paypal.com/en_US/i/btn/x-click-but01.gif'""")

    paypal_button_alt = Attribute("""Paypal Button Alt Text

    o Alt text for the Paypal button.

    Default value:
        "Make payments with PayPal - it's fast, free and secure!" """)

    tool_key = Attribute("""Tool Key

    o Salt used to prevent spoofing of requests by hashing item #s.

    o Read-only.

    Default value: '', replaced during __init__ with a randomly-generated
                       hex string.
    """)

    def makePaypalButton(currency=None,
                         cart=None,
                         invoice=None,
                         return_url=None,
                         cancel_url=None,
                         custom=None,
                        ):
        """ Return an HTML form which posts to PayPal using properties.

        o 'currency', if passed, overrides the 'default_currency' for the
          transaction (e.g., pass 'USD' to use dollars).

        o 'cart', if passed, should be a sequence of 3-tuples,
          (item_name, price, quantity), representing line items purchased.

        o 'invoice', if passed, is a site-generated ID for the transaction.
          If not passed, no value will be posted to PayPal.

        o 'return_url', if passed, is the URL of the page to which Paypal
          will redirect after successfully processing the payment;  default
          is the tool's 'default_return_path'.

        o 'return_url', if passed, is the URL of the page to which Paypal
          will redirect after cancelling the payment;  default
          is the tool's 'default_cancel_path'.

        o 'custom', if passed, will be passed on to Paypal as a site-generated
          description for the transaction.
        """

    def paypal_notification(REQUEST):
        """ Process a paypal notification callback.

        o TODO:  Explain the mechanism here.
        """

    def checkVerificationKey(item, key):
        """ Check that a key and item number match, using our 'tool_key'.
        """

    def updateProperties(REQUEST):
        """ Update tool configuration.
        """
