""" Unit tests for CMFPaypalHelper.PaypalTool

$Id: test_PaypalTool.py 41349 2007-04-30 19:15:50Z tseaver $
"""
import unittest

class PaypalToolTests(unittest.TestCase):

    def _getTargetClass(self):
        from Products.CMFPaypalHelper.PaypalTool import PaypalTool
        return PaypalTool

    def _makeOne(self, *args, **kw):
        return self._getTargetClass()(*args, **kw)

    def test_class_conforms_to_IPaypalTool(self):
        from zope.interface.verify import verifyClass
        from Products.CMFPaypalHelper.interfaces import IPaypalTool
        verifyClass(IPaypalTool, self._getTargetClass())

    def test_instance_conforms_to_IPaypalTool(self):
        from zope.interface.verify import verifyObject
        from Products.CMFPaypalHelper.interfaces import IPaypalTool
        verifyObject(IPaypalTool, self._makeOne())

    def test_construction_creates_base64_tool_key(self):
        from base64 import decodestring
        tool = self._makeOne()
        self.assertEqual(len(decodestring(tool.tool_key)), 16)

    def test_makePaypalButton_defaults(self):
        from OFS.Folder import Folder
        _EXPECTED = """\
<form name="_xclick" method="post" action="https://www.sandbox.paypal.com/cgi-bin/webscr" >
<input type="hidden" name="cmd" value="_cart" />
<input type="hidden" name="upload" value="1" />
<input type="hidden" name="no_shipping" value="1" />
<input type="hidden" name="business" value="account@example.com" />
<input type="hidden" name="return" value="http://example.com/site/index_html?portal_status_message=Payment+accepted." />
<input type="hidden" name="cancel_url" value="http://example.com/site/index_html?portal_status_message=Payment+cancelled." />
<input type="hidden" name="currency_code" value="EUR" />
<input type="hidden" name="notify_url" value="http://example.com/site/portal_paypal/paypal_notification" />

<input type="image" src="http://www.paypal.com/en_US/i/btn/x-click-but01.gif" border="0" name="submit"
       alt="Make payments with PayPal - it's fast, free and secure!" />
</form>"""
        site = Folder()
        site._setId('site')
        site.REQUEST = DummyRequest()
        site.portal_url = DummyURLTool(site)
        site.absolute_url = lambda *x: 'http://example.com/site'
        tool = self._makeOne().__of__(site)
        self.assertEqual(tool.makePaypalButton(), _EXPECTED)

    def test_makePaypalButton_overrides_no_cart(self):
        from OFS.Folder import Folder
        _EXPECTED = """\
<form name="_xclick" method="post" action="https://www.sandbox.paypal.com/cgi-bin/webscr" >
<input type="hidden" name="cmd" value="_cart" />
<input type="hidden" name="upload" value="1" />
<input type="hidden" name="no_shipping" value="1" />
<input type="hidden" name="business" value="account@example.com" />
<input type="hidden" name="return" value="http://after.example.com/site/yes" />
<input type="hidden" name="cancel_url" value="http://after.example.com/site/no" />
<input type="hidden" name="currency_code" value="USD" />
<input type="hidden" name="notify_url" value="http://example.com/site/portal_paypal/paypal_notification" />
<input type="hidden" name="invoice" value="XXX" />
<input type="hidden" name="custom" value="YYY" />
<input type="image" src="http://www.paypal.com/en_US/i/btn/x-click-but01.gif" border="0" name="submit"
       alt="Make payments with PayPal - it's fast, free and secure!" />
</form>"""
        site = Folder()
        site._setId('site')
        site.REQUEST = DummyRequest()
        site.portal_url = DummyURLTool(site)
        site.absolute_url = lambda *x: 'http://example.com/site'
        tool = self._makeOne().__of__(site)
        _URL = 'http://after.example.com/site'
        self.assertEqual(tool.makePaypalButton(currency='USD',
                                               invoice='XXX',
                                               return_url='%s/yes' % _URL,
                                               cancel_url='%s/no' % _URL,
                                               custom='YYY',
                                              ), _EXPECTED)

    def test_makePaypalButton_with_cart_and_invoice(self):
        from OFS.Folder import Folder
        _EXPECTED = """\
<form name="_xclick" method="post" action="https://www.sandbox.paypal.com/cgi-bin/webscr" >
<input type="hidden" name="cmd" value="_cart" />
<input type="hidden" name="upload" value="1" />
<input type="hidden" name="no_shipping" value="1" />
<input type="hidden" name="business" value="account@example.com" />
<input type="hidden" name="return" value="http://example.com/site/index_html?portal_status_message=Payment+accepted." />
<input type="hidden" name="cancel_url" value="http://example.com/site/index_html?portal_status_message=Payment+cancelled." />
<input type="hidden" name="currency_code" value="EUR" />
<input type="hidden" name="notify_url" value="http://example.com/site/portal_paypal/paypal_notification" />
<input type="hidden" name="item_name_1" value="item_one" />
<input type="hidden" name="amount_1" value="1.00" />
<input type="hidden" name="quantity_1" value="42" />
<input type="hidden" name="item_name_2" value="item_two" />
<input type="hidden" name="amount_2" value="3.14" />
<input type="hidden" name="quantity_2" value="12" />
<input type="hidden" name="invoice" value="XXX" />
<input type="image" src="http://www.paypal.com/en_US/i/btn/x-click-but01.gif" border="0" name="submit"
       alt="Make payments with PayPal - it's fast, free and secure!" />
</form>"""
        site = Folder()
        site._setId('site')
        site.REQUEST = DummyRequest()
        site.portal_url = DummyURLTool(site)
        site.absolute_url = lambda *x: 'http://example.com/site'
        tool = self._makeOne().__of__(site)
        self.assertEqual(tool.makePaypalButton(invoice='XXX',
                                               cart=[('item_one', 1.00, 42),
                                                     ('item_two', 3.14, 12)]
                                              ), _EXPECTED)

    def test_paypal_notification_account_mismatch_raises_RuntimeError(self):
        tool = self._makeOne()
        request = DummyRequest(invoice='XXX',
                               custom='YYY',
                               receiver_email='nonesuch@example.com')

        self.assertRaises(RuntimeError, tool.paypal_notification, request)

    def test_paypal_notification_not_verified_raises_RuntimeError(self):
        from zope.app.testing import ztapi
        from zope.interface import directlyProvides
        from Products.CMFPaypalHelper.interfaces import IPaypalVerifyCallback

        _pvc_called = {}
        def _nope(**kw):
            _pvc_called.update(kw)
            return False, 'You lose'
        directlyProvides(_nope, IPaypalVerifyCallback)
        ztapi.provideUtility(IPaypalVerifyCallback, _nope)
            
        tool = self._makeOne()
        request = DummyRequest(invoice='XXX',
                               custom='YYY',
                               receiver_email='account@example.com')

        self.assertRaises(RuntimeError, tool.paypal_notification, request)
        self.assertEqual(_pvc_called['invoice'], 'XXX')

    def test_paypal_notification_verified_calls_utility(self):
        from OFS.Folder import Folder
        from zope.app.testing import ztapi
        from zope.interface import directlyProvides
        from zope.interface import implements
        from Products.CMFPaypalHelper.interfaces import IPaypalVerifyCallback
        from Products.CMFPaypalHelper.interfaces import IPaypalProcessor

        _pvc_called = {}
        def _sure(**kw):
            _pvc_called.update(kw)
            return True, 'Why not?'
        directlyProvides(_sure, IPaypalVerifyCallback)
        ztapi.provideUtility(IPaypalVerifyCallback, _sure)

        _script_called = {}

        class _Processor:
            implements(IPaypalProcessor)
            def __init__(self, context):
                pass
            def __call__(self, verification_key, payment_info):
                _script_called['verification_key'] = verification_key
                _script_called['payment_info'] = payment_info
        ztapi.provideAdapter(None, IPaypalProcessor, _Processor)
 
        site = Folder()
        site._setId('site')
        site.acl_users = Folder()
        site.portal_url = DummyURLTool(site)
        tool = self._makeOne().__of__(site)
        request = DummyRequest(invoice='XXX',
                               custom='YYY',
                               receiver_email='account@example.com')

        tool.paypal_notification(request)
        self.assertEqual(_pvc_called['invoice'], 'XXX')
        self.assertEqual(_script_called['payment_info']['invoice'], 'XXX')

    def test_paypal_notification_verified_calls_script_as_fallback(self):
        from OFS.Folder import Folder
        from zope.app.testing import ztapi
        from zope.interface import directlyProvides
        from Products.CMFPaypalHelper.interfaces import IPaypalVerifyCallback

        _pvc_called = {}
        def _sure(**kw):
            _pvc_called.update(kw)
            return True, 'Why not?'
        directlyProvides(_sure, IPaypalVerifyCallback)
        ztapi.provideUtility(IPaypalVerifyCallback, _sure)

        _script_called = {}
        def _script(**kw):
            _script_called.update(kw)
 
        site = Folder()
        site._setId('site')
        site.acl_users = Folder()
        site.portal_url = DummyURLTool(site)
        site.validatePaypalPayment = _script
        tool = self._makeOne().__of__(site)
        request = DummyRequest(invoice='XXX',
                               custom='YYY',
                               receiver_email='account@example.com')

        tool.paypal_notification(request)
        self.assertEqual(_pvc_called['invoice'], 'XXX')
        self.assertEqual(_script_called['payment_info']['invoice'], 'XXX')

class DummyRequest:

    def __init__(self, **kw):
        self.form = dict(kw)

    def physicalPathToURL(self, path):
        return 'http://example.com/%s' % '/'.join(path)

class DummyURLTool:

    def __init__(self, site):
        self.site = site

    def __call__(self):
        return 'http://example.com/site'

    def getPortalObject(self):
        return self.site

def test_suite():
    return unittest.TestSuite((
        unittest.makeSuite(PaypalToolTests),
        ))

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')
